/*
 * Created on Oct 27, 2004
 *
 */
package sync;

import java.util.*;

/**
 * An object of this class simulates a receiver of messages. After it has
 * received a message, it delays for a fixed amount of time (simulating the 
 * work on the message). Thereby, it has a fixed receive rate.
 * <p>
 * @see Sender Sender for an example<p>
 * @author hvogt<p>
 *
 */
public class Receiver implements Runnable {

	SyncPort sp;
	
	long waitingTime = 0;
	
	int delay = 250;
	
	List data = new ArrayList();
	
	/**
	 * The constructor for receiver objects.
	 * 
	 * @param p The port over which the receiver will receive its messages.
	 */
	public Receiver(SyncPort p) {
		sp = p;
		new Thread(this).start();
	}
	
	/**
	 * The thread loop.
	 */
	public void run() {
	    long t0 = System.currentTimeMillis();
		try {
			run_ex();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
		long t1 = System.currentTimeMillis();
		System.out.println("Receiver total running time: "+(t1-t0)+" ms");
		System.exit(0);
	}
	
	void run_ex() throws Exception {
		boolean fini = false;
		
		while (!fini) {
			long t0 = System.currentTimeMillis();
			int n = sp.receive();
			long t1 = System.currentTimeMillis();
			long blocked = t1 - t0;
			waitingTime += blocked;
			System.out.println("\t\t\t\tRECEIVE("+n+") blocked for "+blocked+" ms");
			data.add(new Integer(n));
			Thread.sleep(delay);
			if (n == 9999) {
			    fini = true;
			}
		}
		System.out.println("Receiver total blocked time: "+waitingTime+" ms");
		System.out.print("Received the following data: ");
		for (int i = 0; i < data.size(); i++) {
		    System.out.print(data.get(i)+" ");
		}
		System.out.println();
	}
	
}
